@@ -3,4 +3,4 @@ Speckle Tracking scans. Wavefront propagation is based on
 the Fresnel diffraction theory.
 """
 from .st_sim_param import STParams, parameters
-from .st_sim import STSim, STConverter
\ No newline at end of file
+from .st_sim import STSim, STConverter, converter
